//
//  StartStopp.m
//  HeisApp
//
//  Created by Sturla Sandvei on 4/21/11.
//  Copyright 2011 NTNU. All rights reserved.
//

#import "StartStopp.h"
#import "HeisAppDelegate.h"

@implementation StartStopp

@synthesize startTid;

// Tar bort kvitteringsmelding på skjermen etter at timeren har gått ut
- (void)stopp:(NSTimer *)mldTimer {
	[[mldTimer userInfo] dismissWithClickedButtonIndex:0 animated:YES];
	[mldTimer invalidate];
}

// Rutine som trigges når brukeren trykker på knappen for start/stopp av måling
- (IBAction)startStopp:(id)sender
{    
    HeisAppDelegate *hovedDelegat = (HeisAppDelegate *)[[UIApplication sharedApplication] delegate];

    // Sjekk om målingen er aktiv eller ikke, dvs om dette er en start- eller stopp-ordre
    if (hovedDelegat.sensorAktiv)
    {
        // Sensoren var allerede aktiv - og da må vi stoppe måingene og lagre resultatet
        [self stoppForsok: hovedDelegat];
        
        // Bytt ut knappen på display til å vise start-signal igjen
        [sender setBackgroundImage:[UIImage imageNamed:@"Play1Normal.png"] forState:UIControlStateNormal];
    }
    else
    {
        // Sensoren er ikke aktiv - og da må vi starte sensorene og måle-loopen
        [self startForsok: hovedDelegat];

        // Bytt ut knappen på display til å vise stopp-signal        
        [sender setBackgroundImage:[UIImage imageNamed:@"Stop1NormalRed.png"] forState:UIControlStateNormal];
    }
}

// Her har brukeren valgt å starte et nytt forsøk
- (void) startForsok:(HeisAppDelegate *)hovedDelegat
{    
    UIAlertView *melding;
    NSDate *idag = [[NSDate alloc] init];
    NSDateFormatter *tidsFormat = [[NSDateFormatter alloc] init];

    // Opprett en instans av CMMotionManager
    motionManager = [[CMMotionManager alloc] init];
    
    // Oppdateringsintervall angis i sekunder slik at 0.005 tilsvarer 200 Hz.
    motionManager.deviceMotionUpdateInterval = 0.005;
    
    // Start opp sensorene dvs gyro og aksellerometer
    [motionManager startDeviceMotionUpdates];
    hovedDelegat.sensorAktiv = 1;
    antReg = 0;
    diff = hovedDelegat.differanse; // Hvis kalibrering er kjørt av bruker
    
    // Starttidspunkt lagres som hh:mm:ss
    [tidsFormat setDateFormat:@"HH:mm:ss"];
    self.startTid = [tidsFormat stringFromDate:idag];
    
    // Setter opp programløkken som skal gå i metoden "registrerData" med angitt intervall, 0.0125 sek (80 Hz)
    // Intervall litt mer enn det doble av frekvensen på devMotion så vi slipper doble målinger.
    timer = [[NSTimer scheduledTimerWithTimeInterval:0.0125 target:self selector:@selector(registrerData) 
                                            userInfo:nil repeats:YES] retain];
    // Legger ut kvittering til skjerm
    melding = [[UIAlertView alloc]
               initWithTitle:@"Resultat" message:@"Målingene er nå startet" 
               delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];

    // Kvitteringen forsvinner etter 2 sekunder med mindre brukeren rekker å trykke OK først
    [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(stopp:) userInfo:melding repeats:NO];
    
    [melding show];
    [melding release];
    
    [idag release];
    [tidsFormat release];
}

// Her har brukeren valgt å stoppe et forsøk som er aktivt
- (void) stoppForsok:(HeisAppDelegate *) hovedDelegat
{
    
    UIAlertView *melding;
    NSDate *idag = [[NSDate alloc] init];
    NSDateFormatter *datoFormat = [[NSDateFormatter alloc] init];
    NSDateFormatter *tidsFormat = [[NSDateFormatter alloc] init];
    
    // Sensoren var allerede aktiv - og da må vi stoppe måingene og lagre resultatet
    
    // Stopp programløkken for å lese av verdier fra sensorene
    float intervall = [timer timeInterval]; // Les av intervall da det kan endres pga bufferfylling
    [timer invalidate];
    [timer release];
    
    // Stopp sensorene dvs gyro og aksellerometer
    [motionManager stopDeviceMotionUpdates];
    hovedDelegat.sensorAktiv = 0;
    [motionManager release];
    
    // Les dato og tidspunkt
    
    // Dato og klokke-format for filnavn
    [datoFormat setDateFormat:@"yyyy-MM-dd"];
    NSString *dato = [datoFormat stringFromDate:idag];
    
    NSArray *kataloger = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDirectory, YES);
    NSString *katalog = [kataloger objectAtIndex:0]; // index 0 inneholder stien til appens Documents katalog
    
    // Konstruerer filnavn ut fra dato og klokkeslett med filtype csv
    NSString *csvfilNavn = [NSString stringWithFormat:@"%@_%@.csv",dato, self.startTid]; // tid];
    // Aktiv CSV-fil settes og legges inn i global filliste
    hovedDelegat.csvFil = [katalog stringByAppendingPathComponent:csvfilNavn];
    [hovedDelegat.filTabell addObject:csvfilNavn];
    
    // Lagre datatabell med måleverdier inn i datafilen
    if (antReg != 0) {
        // Et annet datoformat skal lagres inne i CSV-filen            
        [datoFormat setDateFormat:@"dd.MM.yyyy"];
        [tidsFormat setDateFormat:@"HH:mm:ss"];
        NSString *sluttTid = [tidsFormat stringFromDate:idag];
        // NSString *dato = [datoFormat stringFromDate:idag];
        
        // Putter alle linjene som skal inn i CSV-filen inn i en tabell først. Heading først
        NSMutableArray *csvLinjer = [[NSMutableArray alloc] init];
        NSString *enLinje;
        enLinje = [NSString stringWithFormat:@"Heisforsøk,iPhone,\n"];
        [csvLinjer insertObject:enLinje atIndex:0];
        
        // Mere headings for CSV-filen, dato, start- og stopp-tid og kolonneoverskrifter
        enLinje = [NSString stringWithFormat:@"Dato:,%@,\n", dato];
        [csvLinjer insertObject:enLinje atIndex:1];
        enLinje = [NSString stringWithFormat:@"Start:,%@,\n", self.startTid];
        [csvLinjer insertObject:enLinje atIndex:2];
        enLinje = [NSString stringWithFormat:@"Stopp:,%@,\n", sluttTid];
        [csvLinjer insertObject:enLinje atIndex:3];
        enLinje = @"Sekunder,Nummer,Sum krefter,\n";
        [csvLinjer insertObject:enLinje atIndex:4];
        
        double snittAks = 0;
        
        // Går gjennom alle målingne som er registrert og snitter hver 8. måling
        // før lagring til tabellen. Dette for å krympe data litt og for å unngå peak-målinger.
        for (int i=0; i < antReg; i++) 
        {
            snittAks += acc[i];
            // Vi skriver til filen bare hver femte gang
            if ( (i % 8) == 7 ) 
            {
                // Skriver inn antall sekunder siden start, målingsnummer og selve verdien
                enLinje = [[NSString alloc] initWithFormat:@"%f,%i,%f,\n", (i+1)*intervall, (i+1)/8, snittAks/8];
                [csvLinjer insertObject:enLinje atIndex:((i+1)/8)+4];
                snittAks = 0;
            }
        }
        
        // Putter hele tabellen inn i CSV-filen som lagres i telefonens minne
        [[csvLinjer componentsJoinedByString:@""] writeToFile:hovedDelegat.csvFil atomically:YES
                                                     encoding:NSUTF8StringEncoding error:nil];
        
        [csvLinjer release];
        
    }
    
    // Legg ut kvittering til skjermen
    melding = [[UIAlertView alloc]
               initWithTitle:@"Resultat" message:@"Målingene er nå stoppet" 
               delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    
    // Kvitteringen forsvinner etter 2 sekunder med mindre brukeren rekker å trykke OK først
    [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(stopp:) userInfo:melding repeats:NO];
    
    [melding show];
    [melding release];
    
    [idag release];
    [tidsFormat release];
    [datoFormat release];

}

// Rutine som kalles kontinuerlig når målingen er aktiv
- (void) registrerData
{
    // Her kommer vi i loop etter at målingene er startet
    // Eneste oppgave er å lese fra sensor og lagre i tabell
    CMAcceleration gravitasjon = motionManager.deviceMotion.gravity;
    CMAcceleration useracc = motionManager.deviceMotion.userAcceleration;

    /*
     Målingene fra DeviceMotion angis i antall G, dvs 1.0 tilsvarer 1G .
    
     Det er Newtons andre lov vi er ute etter akkurat i denne appen (Summen av kreftene som virker på et legeme er
     lik legemets masse multiplisert med akselerasjonen. Dvs summen er 0 hvis akselerasjonen er 0 (konst hastighet).
     
     F = m * a
            m = 137 gram
            G = 9.81 m/s/s
     */
    
    // Det tar noen runder å komme igang med målingene så vi venter til G registreres av DevMotion
    if ((gravitasjon.z > 0.000001) || (gravitasjon.z < -0.000001))
    {
        // Lagrer brukerakselerasjon i datatabell evt korrigert med diff funnet ved kalibrering
        acc[antReg] = ( diff + useracc.z ) * 9.81;
        antReg++;            
    }

    
    // Bufferet har plass til 250000 målinger noe som betyr en varighet på (0.0125*250000) = 3125 sek = 52 min.
    // Men noen kan jo glemme å skru av målingene eller de vil testet "noe" som tar en time...
    // Derfor tar vi med en sjekk på om bufferet akkurat er fyllt opp slik at aplikasjonen ikke crasher!
    if (antReg == 250000)
    // Bufferet er fullt! Da pakker vi det sammen ved å slå sammen to og to målinger som snittes.
    {
        // Les av gammel timerverdi og stopp timer som kjører denne loopen
        float intervall = [timer timeInterval];
        [timer invalidate];

        // Gå gjennom hele bufferet og slå sammen to og to verdier
        float toSum=0;
        for (int i=0; i<antReg; i++)
        {
            toSum += acc[i];
            // Hvis det er andre, fjerde, sjette osv må vi snitte og lagre
            if (i % 2 == 1)
            {
                acc[i/2] = toSum/2;
                toSum = 0;
            }
        }
        
        // Antall registreringer er nå halvvert
        antReg = antReg/2;
        
        // Start opp timeren for å få igang loopen igjen. For at granulariteten for kommende
        // målinger skal være lik det vi nå har pakket i bufferet må intervallet dobles.
        timer = [[NSTimer scheduledTimerWithTimeInterval:intervall*2 target:self selector:@selector(registrerData) 
                                                userInfo:nil repeats:YES] retain];
    }


}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}


- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc. that aren't in use.
}


- (void)viewDidUnload
{
    [super viewDidUnload];
    
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}


- (void)dealloc
{
    [super dealloc];
}

@end
