//
//  Verktoy.m
//  HeisApp
//
//  Created by Sturla Sandvei on 4/22/11.
//  Copyright 2011 NTNU. All rights reserved.
//

#import "Verktoy.h"

@implementation Verktoy

// Tar bort kvitteringsmelding på skjermen etter at timeren har gått ut
- (void)stopp:(NSTimer *)mldTimer {
	[[mldTimer userInfo] dismissWithClickedButtonIndex:0 animated:YES];
	[mldTimer invalidate];
    
}

// Kalibrere målingene slik at de viser (ca) 0 når telefonen ligger rolig
// Kalles når brukeren har trykt på knappen for å kalibrere sensorer
- (IBAction)kalibrer:(id)sender
{
    HeisAppDelegate *hovedDelegat = (HeisAppDelegate *)[[UIApplication sharedApplication] delegate];
    
    UIAlertView *melding;

    // Sjekker først om målingen kanskje allerede er aktiv
    if (hovedDelegat.sensorAktiv)
	{
        melding = [[UIAlertView alloc]
                                initWithTitle:@"** Feilmelding **" message:@"Stopp målingen før kalibrering" 
                                delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
    }
    else
    {
        CMMotionManager *motionManager = [[CMMotionManager alloc] init];
		motionManager.deviceMotionUpdateInterval = 0.02;
        [motionManager startDeviceMotionUpdates];
        hovedDelegat.sensorAktiv = 1;
        
        // Vent litt slik at sensorene garantert har startet opp
        sleep(1);
        
        float diff = 0.0;
        
        // Les av 100 målinger
        for (int i=0; i < 100; i++) {
            CMAcceleration useracc = motionManager.deviceMotion.userAcceleration;
            diff += useracc.z;
        }
        
        // Snitte de 100 målingene
        hovedDelegat.differanse = -diff/100;
        
        // Stopp sensorene
        [motionManager stopDeviceMotionUpdates];
        [motionManager release];
        hovedDelegat.sensorAktiv = 0;
        
        melding = [[UIAlertView alloc]
                                initWithTitle:@"Resultat" message:@"Kalibrering er nå utført" 
                                delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        
    }
    
    // Starter en timer for å ta bort meldingen hvis ikke brukeren trykker OK
    [NSTimer scheduledTimerWithTimeInterval:2 target:self selector:@selector(stopp:) userInfo:melding repeats:NO];
    
    [melding show];
    [melding release];
    
}

// Kalles når brukeren har trykket på knappen for å sende CSV-fil med epost
- (IBAction)sendMail:(id)sender
{
    // Sjekk først hva apparatet er kapabelt til
    if ( [MFMailComposeViewController canSendMail] )
        [self visEpostOppsett];
    // Ikke aktuelt med else-delen her da vi kjører iPhone4 - iOS4.
    // Alternativet for eldre iOS er å bruke en mailto-løsning som ikke gir mulighet
    // til å legge med et filvedlegg. Dvs det er uansett ikke interessant.
        
}

// iOS som støtter epost-sending inne i applikasjonen...
// Da får vi også sendt med en fil som vedlegg :-)
-(void) visEpostOppsett
{
    HeisAppDelegate *hovedDelegat = (HeisAppDelegate *)[[UIApplication sharedApplication] delegate];
    
    // Sjekk om det er satt en aktiv fil. Vi ønsker ikke å sende en tom epost...
    if ( [hovedDelegat.csvFil isEqualToString:@"ingen"] )
        // Her er det ikke valgt noen aktiv CSV-fil så vi legger ut en feilmelding
    {
        UIAlertView *melding = [[UIAlertView alloc]
                                initWithTitle:@"** Feilmelding **" message:@"Det er ikke valgt CSV-fil å sende" 
                                delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        
        // Starter en timer for å ta bort meldingen hvis ikke brukeren trykker OK
        [NSTimer scheduledTimerWithTimeInterval:2 target:self selector:@selector(stopp:) userInfo:melding repeats:NO];
        
        [melding show];
        [melding release];
    }
    else
        // Det var satt en aktiv fil slik at vi kan sende epost
    {
        MFMailComposeViewController *epost= [[MFMailComposeViewController alloc] init];
        epost.mailComposeDelegate = self;
        
        // NSArray *mottakere = [NSArray arrayWithObjects:@"sturlsa@stud.ntnu.no", @"sturla@netcom.no", nil];
        //[epost setToRecipients:mottakere];
        [epost setSubject:@"Epost fra iPhone HeisApp"];
        [epost setMessageBody:@"Denne meldingen er sendt fra en iPhone testapplikasjon." isHTML:YES];

        // Aktiv fil ligger lagret i global variabel csvFil
        NSData *vedlegg = [NSData dataWithContentsOfFile:hovedDelegat.csvFil];
        [epost addAttachmentData:vedlegg mimeType:@"text/csv" fileName:@"datafil.csv"];
        
        // Sort stil passer med resten av layout
        epost.navigationBar.barStyle = UIBarStyleBlack;
        
        // Åpne vindu med standard grensesnitt for å sende epost
        [self presentModalViewController:epost animated:YES];
        
        [epost release];        
    }
}

- (void)mailComposeController:(MFMailComposeViewController *)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError *)error
{
    UIAlertView *melding;
    
    // Komponer en kvitteringsmelding ut fra resultatet av epost-sending
    switch (result) {
        case MFMailComposeResultCancelled:
            melding = [[UIAlertView alloc] initWithTitle:@"Epost" message:@"Sendingen ble kansellert..." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil]; 
            break;
        case MFMailComposeResultSaved:
            melding = [[UIAlertView alloc] initWithTitle:@"Epost" message:@"Meldingen er lagret..." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
            break;
        case MFMailComposeResultSent:
            melding = [[UIAlertView alloc] initWithTitle:@"Epost" message:@"Meldingen er sendt..." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
            break;
        case MFMailComposeResultFailed:
            melding = [[UIAlertView alloc] initWithTitle:@"Epost" message:@"Sendingen feilet..." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
            break;
        default:
            melding = [[UIAlertView alloc] initWithTitle:@"Epost" message:@"Sendingen har trolig feilet..." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
            break;
    }
    
    // Start en timer som tar bort kvitteringen fra skjermen med mindre brukeren trykker på OK
    [NSTimer scheduledTimerWithTimeInterval:1 target:self selector:@selector(stopp:) userInfo:melding repeats:NO];
    
    [melding show];
    [melding release];
    
    // Gå tilbake til appens eget view
    [self dismissModalViewControllerAnimated:YES];
}


- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)dealloc
{
    [super dealloc];
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.
}

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

@end
